#include <avr/io.h>
#include <stdio.h>
#include <util/delay.h>

// this is included from AVRlib (we need to link with uart2.c and buffer.c)
#include <uart2.h>

// Size of internal line buffer used by uart1_getchar()
#define RX_BUFSIZE 80

static int uart1_putchar(char c, FILE *stream);
static int uart1_getchar(FILE *stream);
static FILE uart_stdinout = FDEV_SETUP_STREAM(uart1_putchar, uart1_getchar, _FDEV_SETUP_RW);

//outputs one character to UART1
static int uart1_putchar(char c, FILE *stream)
{
  //we live in Windows world now 
  if (c == '\n') uart1_putchar('\r', stream);
  uart1SendByte(c);
  return 0;
}

// Receive a character from the UART Rx with simple line-editor
// Editing characters:
//  Backspace, DEL, ^u (del line), ^w (del word), ^r (refresh), tabs replaced by spaces
// Borrowed from WinAVR example stdiodemo
int uart1_getchar(FILE *stream)
{
  uint8_t c;
  char *cp, *cp2;
  static char b[RX_BUFSIZE];
  static char *rxp;

  if (rxp == 0)
    for (cp = b;;)
      {
	
	do { c = uart1GetByte();} while (c==(char)-1);
	
	/* behaviour similar to Unix stty ICRNL */
	if (c == '\r')
	  c = '\n';
	if (c == '\n')
	  {
	    *cp = c;
	    uart1_putchar(c, stream);
	    rxp = b;
	    break;
	  }
	else if (c == '\t')
	  c = ' ';

	if ((c >= (uint8_t)' ' && c <= (uint8_t)'\x7e') ||
	    c >= (uint8_t)'\xa0')
	  {
	    if (cp == b + RX_BUFSIZE - 1)
	      uart1_putchar('\a', stream);
	    else
	      {
		*cp++ = c;
		uart1_putchar(c, stream);
	      }
	    continue;
	  }

	switch (c)
	  {
	  case 'c' & 0x1f:
	    return -1;

	  case '\b':
	  case '\x7f':
	    if (cp > b)
	      {
		uart1_putchar('\b', stream);
		uart1_putchar(' ', stream);
		uart1_putchar('\b', stream);
		cp--;
	      }
	    break;

	  case 'r' & 0x1f:
	    uart1_putchar('\r', stream);
	    for (cp2 = b; cp2 < cp; cp2++)
	      uart1_putchar(*cp2, stream);
	    break;

	  case 'u' & 0x1f:
	    while (cp > b)
	      {
		uart1_putchar('\b', stream);
		uart1_putchar(' ', stream);
		uart1_putchar('\b', stream);
		cp--;
	      }
	    break;

	  case 'w' & 0x1f:
	    while (cp > b && cp[-1] != ' ')
	      {
		uart1_putchar('\b', stream);
		uart1_putchar(' ', stream);
		uart1_putchar('\b', stream);
		cp--;
	      }
	    break;
	  }
      }

  c = *rxp++;
  if (c == '\n')
    rxp = 0;

  return c;
}


//this function sleeps s seconds (if F_CPU is set to frequency of processor)
void sleep(uint8_t s)
{
  uint8_t i;
  while (s--) for (i = 0; i < 100; i++) _delay_ms(10);
}

int main(void)
{
  int a, b, c;

  //set port E as output so that we can operate with LEDs
  DDRE = 0xFF;  
  
  //first LED shows that program runs
  PORTE = PINE | 16;
  
  //initialize UART1 as 2400/odd parity (that's how our BlueSmirf is configured)
  uart1Init();
  uartSetBaudRate(1, 2400);
  UCSR1C |= 48;

  //setup character output function for stdout
  stdout = stdin = &uart_stdinout; 

  //send string to UART1
  printf("AVR calculator.\r\n");
  printf("a=");
  scanf("%d", &a);
  printf("b=");
  scanf("%d", &b);
  c=a+b;
  printf("a+b=%d (0x%x)\r\n", c, c);
    
  //turn off first LED to show that program terminates
  PORTE = PINE & ~16;
  
  return 0;
}
